/* Arithmetic/Logic Unit and Bus Controller
 * Michael Robinson
 */
`timescale 1ns/10ps

/* ALU block.  Handles bus control as well 
 * Input:
 *        t1 = timestate 1: write result register
 *        t23 = timestates 2 and 3: output result to ddbus
 *        t3_ = timestate 3: store result according to dest
 *        reset_ = system-wide reset
 *        mew_ = enable writes to device
 *        run = run switch
 *        write_= write switch
 *        [1:0] dest = destination code
 *           00 = accumulator, 01 = bus device, 10 = indirection register,
 *           11 = page register
 *        i = indirection bit: 0 = use address in instruction 
 *           1 = use indirection register
 *        op = opcode to run
 *        pre_ = front panel selected page register
 *        enrr = enable result output to bus (from front panel)
 * Output:
 *        [7:0] ablo = data address bus
 *        [7:0] abhi = data address bus
 *        f = output flag from ALU
 *        devwr_ = command device to take data
 *        devrd_ = command device to present data
 * Input/Output:
 *        [7:0] ddbus = data bus
 */
module alublk( ddbus, ablo, abhi, t1, t23, t3_, reset_, mew_, run, enrr, 
               write_, dest, op, i, pre_, f, devwr_, devrd_ ); 
inout [7:0] ddbus;
input [7:0] ablo, abhi;
input [3:0] op;
input enrr;
input t1, t23, t3_;
input reset_;
input mew_, run, write_, pre_;

input [1:0] dest;
input i;

output f;
output devwr_;
output devrd_;

wire [7:0] operand1;
wire [7:0] result;

wire devwr_;
wire devrd_;

wire rron;
wire prclk;
wire iroe_;
wire dac_;
wire dac;
wire dme_;
wire dir_;
wire dir;
wire dpr_;
wire dpr;

assign devwr_ = dme_ & ( mew_ | write_ );
assign devrd_ = ~( enrr & rron );

assign rron = ~( t23 & enrr & run );
assign prclk = dpr | ~( write_ | pre_ );
assign iroe_ = ~( i & run );
assign dac = ~dac_;
assign dir = ~dir_;
assign dpr = ~dpr_;

u74ls139 ddecode( .a(dest), .e_(t3_), .q({dpr_, dir_, dme_, dac_}));

u74ls374 ir( .oe_(iroe_), .d(ddbus), .q(ablo), .clk(dir));
u74ls273 ac( .d( ddbus), .q(operand1), .clk(dac), .mr_(reset_));
u74ls374 rr( .oe_(rron), .clk(t1), .d(result), .q(ddbus) );
u74ls273 pr( .d(ddbus), .q(abhi), .clk(prclk), .mr_(reset_));
aluunit alunit( .a(operand1), .b(ddbus), .op(op), .f(result), .bit(f) );

endmodule