/* Computer #4
 * Michael Robinson 
 */

`timescale 1ns/10ps

/* Comp4 processor module (with, at the moment, on board data memory)
 * Input: [7:0] di = data switches
 *        [8:0] ai = address switches.  Bits 0-7 = base address.  Bit 8
 *                   for selecting conditional data address
 *        run = High when processor is running and is bus master
 *        reset_ = Active low system reset
 *        clk = System clock.  Generally triggers stuff on rising edge
 *        step = triggers one clock cycle manually
 *        write_ = Write switch.  Triggers data to be taken into the 
 *                 destination selected by (sre_, pre_, mee_, i1e_, i2e_,
 *                 i3e_)
 *        write_en_ = Active low.  Clears off ddbus so switch data may be
 *                    written to memory.  When high, processor has control
 *                    of ddbus.
 *        sre_ = When halted, switches refer to state register
 *        pre_ = When halted, switches refer to page register
 *        mee_ = When halted, switches refer to memory/devices
 *        i1e_ = When halted, switches refer to first instruction word
 *        i2e_ = When halted, switches refer to second instruction word
 *        i3e_ = When halted, switches refer to third instruction word
 * Output:
 * Bus: 
 *     I  pause_ = When asserted, allows device to pause processor
 *     O  devwr_ = commands device addressed by ablo and abhi to absorb
 *                 data from ddbus
 *     O  devrd_ = commands device addressed by ablo and abhi to place
 *                 data onto ddbus
 *     O  [7:0] ablo = low byte of address
 *     O  [7:0] abhi = high byte of address
 *     IO [7:0] ddbus = data bus
 * Instruction Set:
 *  Three word (8 bits each) instruction
 * 
 * I1: 
 * MSB                   LSB
 * +--+--+--+--+--+--+--+--+
 * | 0|           |     |  |
 * +--+--+--+--+--+--+--+--+
 *   |      |        |    |
 *   |      |        |    \_ I: 0 = use I2 for low 8 bits of address
 *   |      |        |          1 = use indirection register IR
 *   |      |        \______ DEST:  00 = Accumulator  01 = Device/Memory
 *   |      |                       10 = IR           11 = Page register
 *   |      \_______________ OP: Operation (see table)
 *   \______________________ Unused
 *
 * Operations: (Act on values in accumulator and addressed device)
 * -----------
 * 0000 = ACC = Pass accumulator
 * 0001 = DEV = Pass device
 * 0010 = CMP = Bitwise compare ACC and device
 * 0011 = ADD = Arithmatic add
 * 0100 = CLR = Give a zero
 * 0101 = INC = Add 1 to accumulator
 * 0110 = AND = Bitwise AND
 * 0111 = OR  = Bitwise OR
 * 1000 = NOT = Bitwise NOT
 * 1001 = ROL = Rotate left
 * 1010 = ROR = Rotate right
 * 1011 =  = 
 * 1100 =  = 
 * 1101 =  = 
 * 1110 =  = 
 * 1111 =  = 
 * 
 * I2: Address to use to load data from device (not used if I=1)
 * I3: Address of next instruction in program memory
 * 
 * Instruction address format:
 * 
 * MSB                      LSB
 * +--+--+--+--+--+--+--+--+--+
 * |  |                       |
 * +--+--+--+--+--+--+--+--+--+
 *   |              |
 *   |              \__________  Address sent to I1 and I2
 *   \_________________________  Flag bit from ALU, used to address I3
 */
module comp4( di, ai, run, reset_, clk, write_, step, write_en_, sre_, 
             pre_, mee_, i1e_, i2e_, i3e_,
             pause_, devwr_, devrd_, ablo, abhi, ddbus
             );
/* Switch banks */
input [7:0] di;
input [8:0] ai;

/* Run control switches, writing */
input run, step, write_, write_en_;

/* 6 position rotary switch */
input sre_, pre_, mee_, i1e_, i2e_, i3e_;

/* Clocking */
input reset_, clk;

/* Bus I/O */
input pause_;
output devwr_;
output devrd_;
output [7:0] ablo, abhi;
inout [7:0] ddbus;

wire [7:0] aiprg;
wire [7:0] ablo, abhi;
wire [7:0] ddbus;
wire [7:0] dii1;
wire [7:0] dii3;
wire [3:0] op;
wire [1:0] dest;

diroute di_route( .di(di), .pre_(pre_), .i1e_(i1e_), .i2e_(i2e_),
                  .i3e_(i3e_), .sre_(sre_), .mee_(mee_), .write_en_(write_en_),
                  .mew_(mew_), .enrr(enrr), .ddbus(ddbus), .dii1(dii1), 
                  .ablo(ablo), .dii3(dii3) );

airoute ai_route( .ai(ai), .mee_(mee_), .i1e_(i1e_), .i2e_(i2e_),
                  .i3e_(i3e_), .ensr_(ensr_), .ablo(ablo), .aiprg(aiprg) );

sequencer seq( .clk(clk), .reset_(reset_), .run(run), .run_(run_), 
               .pause_(pause_), .t1(t1), .t23(t23), .t3_(t3_), .t4(t4) );

progmem prog_mem ( .sre_(sre_), .i1e_(i1e_), .i2e_(i2e_), .i3e_(i3e_),
                 .mee_(mee_), .run(run), .run_(run_), .write_(write_),
                 .ensr_(ensr_), .aim1(ai[8]), .t1(t1), .t4(t4), .f(f),
                 .aiprg(aiprg), .ablo(ablo), .dest(dest), .op(op), .i(i),
                 .dii1(dii1), .dii3(dii3) );

alublk alu( .ddbus(ddbus), .ablo(ablo), .abhi(abhi), .t1(t1), .t23(t23),
            .t3_(t3_), .reset_(reset_), .mew_(mew_), .run(run), .enrr(enrr),
            .write_(write_), .dest(dest), .i(i), .op(op), .pre_(pre_),
            .f(f), .devwr_(devwr_), .devrd_(devrd_) ); 
endmodule
